﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace memoWI
{
    public partial class Form1 : Form
    {
        string filePath = null;
        SearchDialogBox sdb = null;
        ImgForm _imgForm = null;

        //画像ファイルのファイルパスを格納する
        string imgFilePath;

        //印刷処理に用いる
        int printPosition = 0;
        string printText = "";

        //実行環境がXPかどうかのフラグ
        bool xpFlag = false;
        Dummy dmm;

        //軽快モードのときに用いる変数
        bool maxFlag = false;
        bool lightMode = false;


        public bool XpFlag
        {
            get { return xpFlag; }
        }

        public Dummy Dmm
        {
            get { return dmm; }
            set { dmm = value; }
        }



        public string FilePath
        {
            get { return filePath; }
            set { filePath = value; }
        }

        public string ImgFilePath
        {
            get { return imgFilePath; }
            set { imgFilePath = value; }
        }

        public ImgForm imgForm
        {
            get { return _imgForm; }
            set { _imgForm = value; }
        }

        public TextBox TBox
        {
            get { return textBox1; }
        }


        
        public enum SearRepMode { Search, Replace };
        


        public Form1()
        {
            InitializeComponent();
            this.Icon = Properties.Resources.memoWI;

            //実行環境のOSがXPかどうかを判断。
            OperatingSystem os = Environment.OSVersion;
            if (os.Platform == PlatformID.Win32NT && os.Version.Major == 5 && os.Version.Minor == 1)
            {
                xpFlag = true;
            }
            
        }

        //コマンドラインで引数を指定された場合に呼ばれる。指定されたファイルパスのファイルを開く
        public Form1(string path)
        {
            InitializeComponent();
            this.Icon = Properties.Resources.memoWI;

            //実行環境のOSがXPかどうかを判断。
            OperatingSystem os = Environment.OSVersion;
            if (os.Platform == PlatformID.Win32NT && os.Version.Major == 5 && os.Version.Minor == 1)
            {
                xpFlag = true;
            }

            if (File.Exists(path))
            {
                if (CheckExtension(path))
                {
                    filePath = path;
                    textBox1.Text = File.ReadAllText(path, Encoding.Default);
                    this.Text = Path.GetFileName(filePath) + " - memoWI";
                }
            }
            else
                MessageBox.Show("指定したファイルが見つかりません。", "警告", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

        }

        #region File Menu Methods

        //終了
        private void CloseToolStripMenuItem_Click(object sender, EventArgs e)
        {           
                this.Close();
        }

        //名前を付けて保存
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();            
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            try
            {
                filePath = saveFileDialog1.FileName;
                File.WriteAllText(filePath, textBox1.Text, Encoding.Default);
                textBox1.Modified = false;
                this.Text = Path.GetFileName(filePath) + " - memoWI";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //開く
        private void OpenToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            bool b = DeleteTextAnnounce();

            if(b)
                openFileDialog1.ShowDialog();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            try
            {
                filePath = openFileDialog1.FileName;
                textBox1.Text = File.ReadAllText(openFileDialog1.FileName, Encoding.Default);
                this.Text = Path.GetFileName(filePath) + " - memoWI";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void textBox1_ModifiedChange(object sender, EventArgs e)
        {
            if (textBox1.Modified == true)
                OverwriteToolStripMenuItem.Enabled = true;
            else
                OverwriteToolStripMenuItem.Enabled = false;
        }

        //上書き保存
        private void OverwriteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (filePath != null)
            {
                try
                {
                    File.WriteAllText(filePath, textBox1.Text, Encoding.Default);
                    textBox1.Modified = false;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.ToString(), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
                saveFileDialog1.ShowDialog();
        }

        //新規
        private void NewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            bool b = DeleteTextAnnounce();
            if (b)
            {
                textBox1.Clear();
                textBox1.Modified = false;
                filePath = null;
                this.Text = "無題 - memoWI";
            }           
        }

        public bool DeleteTextAnnounce()
        {
            if (textBox1.Modified)
            {
                DialogResult result;
                string msg = "変更内容は保存されていません。破棄してよろしいでしょうか？";
                result = MessageBox.Show(msg, "警告", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation);
                if (result == DialogResult.Yes)
                    return true;
                else
                    return false;
            }
            else            
                return true;
            
        }

        private void form1_FromClosing(object sender, FormClosingEventArgs e)
        {
            bool b = DeleteTextAnnounce();
            if(!b)
                e.Cancel = true;


            if (!e.Cancel)
            {
                SaveSetting();
            }
        }

        

        #endregion

        #region Edit Menu Methods

        //切り取り
        private void CutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(textBox1.SelectionLength > 0)
                textBox1.Cut();
        }

        //コピー
        private void CopyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
                textBox1.Copy();
        }

        //すべて選択
        private void AllSelectToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if(textBox1.Text.Length == textBox1.SelectionLength)
                textBox1.SelectionLength = 0;
            else
                textBox1.SelectAll();            
        }

        //元に戻す
        private void UndoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (textBox1.CanUndo)
            {
                textBox1.Undo();
                textBox1.ClearUndo();
            }
        }

        //貼り付け
        private void PasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (Clipboard.GetDataObject().GetDataPresent(DataFormats.Text))
                textBox1.Paste();
        }

        //検索
        private void SearchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sdb == null || sdb.IsDisposed)
            {
                sdb = new SearchDialogBox(SearRepMode.Search, textBox1);
                this.AddOwnedForm(sdb);
                if (this.TopMost)
                    sdb.TopMost = true;
                sdb.Show();
            }
        }

        //置換
        private void ReplaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (sdb == null || sdb.IsDisposed)
            {
                sdb = new SearchDialogBox(SearRepMode.Replace, textBox1);
                this.AddOwnedForm(sdb);
                if (this.TopMost)
                    sdb.TopMost = true;
                sdb.Show();                
            }
        }

        //削除
        private void DeleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
            {
                int temp = textBox1.SelectionStart;
                textBox1.Text = textBox1.Text.Remove(textBox1.SelectionStart, textBox1.SelectionLength);
                textBox1.SelectionStart = temp;
                textBox1.Modified = true;
            }
        }

        //削除,切り取り,コピーの有効/無効の切り替え
        private void menuStrip1_MenuActivate(object sender, EventArgs e)
        {
            if (textBox1.SelectionLength > 0)
            {
                DeleteToolStripMenuItem.Enabled = true;
                CutToolStripMenuItem.Enabled = true;
                CopyToolStripMenuItem.Enabled = true;
            }
            else
            {
                DeleteToolStripMenuItem.Enabled = false;
                CutToolStripMenuItem.Enabled = false;
                CopyToolStripMenuItem.Enabled = false;
            }
        }       

        #endregion

        #region Form Menu Methods

        //右端で折り返す
        private void WordWrapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            textBox1.WordWrap = !textBox1.WordWrap;
        }

        //フォント
        private void FontToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = fontDialog1.ShowDialog();

            if (dr == DialogResult.OK)
            {
                textBox1.Font = fontDialog1.Font;
            }
        }

        #endregion

        #region View Menu Methods

        //ステータスバー
        private void statusbarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowStatusBar();
        }

        //ステータスバーの表示
        private void ShowStatusBar()
        {
            statusStrip1.Visible = !statusStrip1.Visible;

            if (statusStrip1.Visible)
            {
                textBox1.Size = new Size(textBox1.Width, textBox1.Height - 23);
                textBox1.Dock = DockStyle.None;
                textBox1.Anchor = (AnchorStyles.Bottom | AnchorStyles.Left |
                    AnchorStyles.Right | AnchorStyles.Top);
            }
            else
            {
                textBox1.Dock = DockStyle.Fill;
            }
        }                       

        //キーが押されたときのイベントハンドラ
        private void textBox1_KeyUp(object sender, KeyEventArgs e)
        {
            StatusLabelChange();

            CountLetter();
        }

        //ステータスバーの行、列ラベルの変更
        private void StatusLabelChange()
        {
            //行の変更            
            ToolStripStatusLabel tss = rowToolStripStatusLabel;
            tss.Text = (textBox1.GetLineFromCharIndex(textBox1.SelectionStart) + 1).ToString() + "行、";
            
            //列の変更
            tss = columnToolStripStatusLabel;
            int temp = textBox1.Text.LastIndexOf("\n", textBox1.SelectionStart);
            int column = textBox1.SelectionStart - temp;
            tss.Text = column.ToString() + "列";
        }

        //テキストボックスをクリックしたときのイベントハンドラ
        private void textBox1_Click(object sender, EventArgs e)
        {
            StatusLabelChange();
        }

        //文字カウント処理
        private void CountLetter()
        {
            int count;
            count = textBox1.Text.Length
                - textBox1.GetLineFromCharIndex(textBox1.Text.Length) * 2;
           
            
            countToolStripStatusLabel.Text = "文字数:" + count;
        }

        #endregion

        #region Tool Menu Methods

        //常に手前表示
        private void TopMostToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (imgForm != null)
            {
                imgForm.TopMost = !imgForm.TopMost;
                if (imgForm.Bg != null)
                    imgForm.Bg.TopMost = !imgForm.Bg.TopMost;
                this.Focus();
            }
            this.TopMost = !this.TopMost;
            
        }

        //オプション
        private void OptionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Option option = new Option(this);
            if (this.TopMost)
                option.TopMost = true;
            DialogResult dr = option.ShowDialog();

            if (dr == DialogResult.OK)
                option.SetOption();
        }

        #endregion

        #region Form_Move_ReSize EventHandler

        //フォームが移動したときのイベントハンドラ
        private void Form1_Move(object sender, EventArgs e)
        {
            if (imgForm != null)
            {
                imgForm.Location = this.Location + new Size(0,2);
                if (imgForm.Bg != null)
                    imgForm.Bg.Location = this.Location;
            }
        }
        
        //フォームがリサイズした時のイベントハンドラ
        private void Form1_SizeChange(object sender, EventArgs e)
        {
            if (!lightMode)
            {
                ResizeForms();
            }
            else
            {
                //最大化の状態からサイズを元に戻すとき用
                if (maxFlag)
                {                    
                    base.OnMove(new EventArgs());
                    OnResizeEnd(new EventArgs());                    
                    maxFlag = false;
                }

                //メインフォームが最大化するときは、軽快モードでもすぐにリサイズする
                if (this.WindowState == FormWindowState.Maximized || this.WindowState == FormWindowState.Minimized)
                {                    
                    ResizeForms();                    
                    Invalidate();
                    PerformLayout();
                    maxFlag = true;
                }


            }
        }

        #endregion

        #region Setting_Read_Write Methods

        //設定情報の読み込み
        private void LoadSetting()
        {
            if (Properties.Settings.Default.SettingEnable)
            {
                this.Location = Properties.Settings.Default.Location;
                this.Size = Properties.Settings.Default.Size;
                textBox1.ForeColor = Properties.Settings.Default.ForeColor;
                textBox1.Font = Properties.Settings.Default.Font;
                textBox1.WordWrap = Properties.Settings.Default.WordWrap;
                if (textBox1.WordWrap)
                    WordWrapToolStripMenuItem.Checked = true;

                statusbarToolStripMenuItem.Checked = Properties.Settings.Default.StatusBarEnable;
                if (statusbarToolStripMenuItem.Checked)
                    this.ShowStatusBar();

                this.TopMost = Properties.Settings.Default.TopMostEnable;
                if (this.TopMost)
                    TopMostToolStripMenuItem.Checked = true;

                this.lightMode = Properties.Settings.Default.LightMode;
                this.maxFlag = Properties.Settings.Default.MaxFlag;
                if (lightMode)
                    LightModeToolStripMenuItem.Checked = true;

                textBox1.BackColor = Properties.Settings.Default.BackColor;


                if (Properties.Settings.Default.BgImgFlag)
                {                   
                    //背景画像に設定した画像ファイルが存在するかどうかを判定
                    if (File.Exists(Properties.Settings.Default.BackImg))
                    {
                        Option option = new Option(this);
                        this.ImgFilePath = Properties.Settings.Default.BackImg;
                        option.ImgBox.Image = new Bitmap(this.ImgFilePath);
                        
                        if (Properties.Settings.Default.Stretch)
                            option.StretchCheck.Checked = true;

                        option.SetImgForm();
                                                
                        //フォームに透明度が設定されていた時                        
                        if (Properties.Settings.Default.FormOpacity != 1)
                        {                            
                            this.Opacity = Properties.Settings.Default.FormOpacity;
                            this.imgForm.Opacity = this.Opacity;
                        }
                        else if (Properties.Settings.Default.ImgOpacity != 1)
                        {                            
                            this.imgForm.SetBackground();
                            this.imgForm.Opacity = Properties.Settings.Default.ImgOpacity;

                            if (xpFlag)
                            {
                                imgForm.Owner = null;
                                imgForm.Bg.AddOwnedForm(imgForm);
                                Dmm.AddOwnedForm(imgForm.Bg);
                            }
                        }

                        option.SetTopMost();

                    }
                    else
                    {
                        MessageBox.Show("背景画像に設定されていた画像ファイルが見つかりませんでした。","情報",MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }                   

                }
                else
                {                   

                    if (Properties.Settings.Default.FormOpacity != 1)
                        this.Opacity = Properties.Settings.Default.FormOpacity;
                }
                
                
            }

            
        }

        //Loadイベントのイベントハンドラ
        private void Form1_Load(object sender, EventArgs e)
        {            
            LoadSetting();
        }

        //設定情報を保存
        private void SaveSetting()
        {
            Properties.Settings.Default["SettingEnable"] = true;
            Properties.Settings.Default["Location"] = this.Location;
            Properties.Settings.Default["Size"] = this.Size;
            Properties.Settings.Default["ForeColor"] = textBox1.ForeColor;
            Properties.Settings.Default["Font"] = textBox1.Font;
            Properties.Settings.Default["WordWrap"] = textBox1.WordWrap;
            Properties.Settings.Default["StatusBarEnable"] = this.statusbarToolStripMenuItem.Checked;
            Properties.Settings.Default["TopMostEnable"] = this.TopMost;
            Properties.Settings.Default["FormOpacity"] = (float)this.Opacity;
            Properties.Settings.Default["LightMode"] = this.lightMode;
            if (this.WindowState == FormWindowState.Maximized)
                Properties.Settings.Default["MaxFlag"] = true;
            else
                Properties.Settings.Default["MaxFlag"] = false;

            Properties.Settings.Default["BackColor"] = textBox1.BackColor;

            if (this.imgForm != null)
            {
                Properties.Settings.Default["BgImgFlag"] = true;
                Properties.Settings.Default["BackImg"] = this.ImgFilePath;
                Properties.Settings.Default["ImgOpacity"] = (float)this.imgForm.Opacity;


                if (this.imgForm.PictureBox1.SizeMode == PictureBoxSizeMode.StretchImage)
                    Properties.Settings.Default["Stretch"] = true;
                else
                    Properties.Settings.Default["Stretch"] = false;
            }
            else
            {
                Properties.Settings.Default["BgImgFlag"] = false;                
            }

            Properties.Settings.Default.Save();
        }

        #endregion

        #region D&D Methods

        //ドラッグアンドドロップされたときに呼ばれる
        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            string[] DDfilePaths = (string[])e.Data.GetData(DataFormats.FileDrop, false);

            string DDfilePath = DDfilePaths[0];

            if (CheckExtension(DDfilePath) && DeleteTextAnnounce())
            {
                filePath = DDfilePath;
                textBox1.Text = File.ReadAllText(DDfilePath, Encoding.Default);
                this.Text = Path.GetFileName(filePath) + " - memoWI";

            }
        }

        //D&Dされたファイルの拡張子を調べて、扱えるファイルだったらtrueを返す
        public bool CheckExtension(string DDfilePath)
        {
            string ex = Path.GetExtension(DDfilePath);

            string[] exArray = {".txt", ".cs", ".vb", ".cpp", ".java", ".js", ".html", ".htm", ".xml", ".vbs", ".csv", ".pl", ".rb", ".xaml" };

            
            foreach (string extension in exArray)
            {
                if (extension == ex)
                    return true;
            }

            return false;
        }

        //D&Dされたファイルがフォームに入ってきたときに呼ばれる
        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            if(e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.All;
            else
                e.Effect = DragDropEffects.None;

        }

        #endregion

        #region Print Methods

        //印刷するときに呼ばれるイベントハンドラ
        private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            if (printPosition == 0)
            {
                printText = textBox1.Text;

                //改行記号を'\n'に統一する
                printText = printText.Replace("\r\n", "\n");
                printText = printText.Replace("\r", "\n");
            }



            Brush brush = new SolidBrush(textBox1.ForeColor);

            int x = e.MarginBounds.Left;
            int y = e.MarginBounds.Top;
            Font printFont = textBox1.Font;

            while (e.MarginBounds.Bottom > y + printFont.Height && printPosition < printText.Length)
            {
                StringBuilder printLine = new StringBuilder("");
                while (true)
                {
                    //印刷する文字が無くなるか、次の文字が改行文字だったら、ループを抜けて印刷する
                    if (printPosition >= printText.Length || printText[printPosition] == '\n')
                    {
                        printPosition++;
                        break;
                    }

                    printLine.Append(printText[printPosition]);

                    //一文字追加してみて印刷幅が超えるかどうか判定
                    if (e.Graphics.MeasureString(printLine.ToString(), printFont).Width > e.MarginBounds.Width)
                    {
                        printLine.Remove(printLine.Length - 1, 1);
                        break;
                    }

                    printPosition++;

                }


                e.Graphics.DrawString(printLine.ToString(), printFont, brush, x, y);

                y += (int)printFont.GetHeight(e.Graphics);


            }

            if (printPosition >= printText.Length)
            {
                e.HasMorePages = false;
                printPosition = 0;
            }
            else
                e.HasMorePages = true;

        }

        //「ページ設定」をクリックしたときに呼ばれるイベントハンドラ
        private void PageSettingToolStripMenuItem_Click(object sender, EventArgs e)
        {            
            pageSetupDialog1.ShowDialog();
        }

        //「印刷プレビュー」をクリックしたときに呼ばれるイベントハンドラ
        private void PreviewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            printPreviewDialog1.ShowDialog();
        }

        //「印刷」をクリックしたときに呼ばれるイベントハンドラ
        private void PrintToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = printDialog1.ShowDialog();

            if (dr == DialogResult.OK)
                printDocument1.Print();
        }

        #endregion

        //リサイズ中は何もしない（軽快モード時）
        protected override void OnResize(EventArgs e)
        {
            if(!lightMode)
                base.OnResize(e);
        }

        //軽快モード時、リサイズが終わったときに画像フォームを移動させる
        private void Form1_ResizeEnd(object sender, EventArgs e)
        {
            if (lightMode)
            {
                Invalidate();
                PerformLayout();
                ResizeForms();
                base.OnMove(new EventArgs());
            }
            
        }

        //各フォームのリサイズを行う
        private void ResizeForms()
        {
            if (imgForm != null)
            {
                imgForm.Size = this.Size - new Size(0,2);
                if (imgForm.Bg != null)
                    imgForm.Bg.Size = this.Size;
            }
        }

        //「軽快モード」
        private void LightModeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            lightMode = !lightMode;            
        }

        //移動中、軽快モードのときは何も行わない
        protected override void OnMove(EventArgs e)
        {    
            if(this.WindowState == FormWindowState.Maximized || !lightMode || this.WindowState == FormWindowState.Minimized)
                base.OnMove(e);
        }

    }
}
